//
// Open Service Platform
// Copyright (c) 2012 Samsung Electronics Co., Ltd.
//
// Licensed under the Apache License, Version 2.0 (the License);
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

/**
 * @file	FIo_ChannelService.cpp
 * @brief	This is the implementation file for the _ChannelService class.
 *
 */

#include <unique_ptr.h>

#include <FBaseStringComparer.h>
#include <FBaseSysLog.h>
#include <FBase_StringConverter.h>
#include <FBaseRt_EventDispatcher.h>
#include "FIo_ChannelService.h"
#include "FIo_ChannelMessages.h"
#include "FIo_IChannelServiceStub.h"

using namespace Tizen::Base;
using namespace Tizen::Base::Collection;
using namespace Tizen::Base::Runtime;
using namespace Tizen::Io;


namespace Tizen { namespace Io
{

_ChannelService::_ChannelService(void)
	: __pIChannelServiceStub(null)
{

}

_ChannelService::~_ChannelService(void)
{

}

result
_ChannelService::Construct(_IChannelServiceStub& stub)
{
	static _StringHashProvider hashProvider;
	static _StringComparer stringComparer;

	__channels.Construct(0, 0, hashProvider, stringComparer);

	__pIChannelServiceStub = &stub;

	__pIChannelServiceStub->SetChannelService(*this);

	return E_SUCCESS;
}

result
_ChannelService::RegisterChannel(const String& channelId, const _IChannelServiceEventListener& listener)
{
	return E_SUCCESS;
}

result
_ChannelService::RegisterChannel(const String& channelId, int clientId)
{
	result r = E_SUCCESS;
	_ChannelInfo* pChannelInfo = null;

	SysLog(NID_IO, "Register a channel : %ls", channelId.GetPointer());

	r = __channels.GetValue(channelId, pChannelInfo);
	if (pChannelInfo != null)
	{
		SysTryReturnResult(NID_IO, pChannelInfo->clientId != clientId, E_SYSTEM,  "Channel has already been registered.");

		SysLog(NID_IO, "Remove garbage values : %ls", channelId.GetPointer());
		__channels.Remove(channelId);
	}

	pChannelInfo = new (std::nothrow) _ChannelInfo;
	SysTryReturnResult(NID_IO, pChannelInfo != null, E_OUT_OF_MEMORY, "The memory is insufficient.");

	pChannelInfo->channelId = channelId;
	pChannelInfo->clientId = clientId;
	pChannelInfo->pGIOChannel = null;

	__channels.Add(channelId, pChannelInfo);

	return E_SUCCESS;
}

result
_ChannelService::UnregisterChannel(const String& channelId)
{
	return E_SUCCESS;
}

result
_ChannelService::UnregisterChannel(int clientId)
{
	SysLog(NID_IO, "Unregister - client =  %d", clientId);

	result r = E_OBJ_NOT_FOUND;

	String key;
	_ChannelInfo* pValue = null;
	IListT<String>* pKeys = __channels.GetKeysN();
	SysTryReturnResult(NID_IO, pKeys != null, E_OUT_OF_MEMORY, "The memory is insufficient.");

	int count = __channels.GetCount();

	for (int i = 0; i < count; i++)
	{
		pKeys->GetAt(i, key);
		__channels.GetValue(key, pValue);
		if (pValue != null && pValue->clientId == clientId)
		{
			SysLog(NID_IO, "Unregister - Channel = %ls", key.GetPointer());
			__channels.Remove(key);
			delete pValue;

			r = E_SUCCESS;
		}
	}

	delete pKeys;

	return r;
}

result
_ChannelService::SendRequest(const String& src,
							 const String& dest,
							 const ArrayList& args,
							 int requestId)
{
	SysLog(NID_IO, "[%ls] ---> [%ls], Request = %d", src.GetPointer(), dest.GetPointer(), requestId);

	_ChannelInfo* pChannelInfo = null;

	__channels.GetValue(dest, pChannelInfo);
	SysTryReturnResult(NID_IO, pChannelInfo != null, E_OBJ_NOT_FOUND,
				"Destination channel is not found.");

	result r = __pIChannelServiceStub->SendRequest(pChannelInfo->clientId, src, dest, args, requestId);
	SysTryReturnResult(NID_IO, r == E_SUCCESS, E_SYSTEM, "Failed to send the request data.");

	return E_SUCCESS;
}

result
_ChannelService::SendNullRequest(const String& src,
							 const String& dest,
							 int requestId)
{
	SysLog(NID_IO, "[%ls] ---> [%ls], Request = %d", src.GetPointer(), dest.GetPointer(), requestId);

	_ChannelInfo* pChannelInfo = null;

	__channels.GetValue(dest, pChannelInfo);
	SysTryReturnResult(NID_IO, pChannelInfo != null, E_OBJ_NOT_FOUND,
				"Destination channel is not found.");

	__pIChannelServiceStub->SendNullRequest(pChannelInfo->clientId, src, dest, requestId);

	return E_SUCCESS;
}

result
_ChannelService::SendResponse(const String& src,
							  const String& dest,
							  const ArrayList& args,
							  int requestId)
{
	SysLog(NID_IO, "[%ls] ---> [%ls], Request = %d", src.GetPointer(), dest.GetPointer(), requestId);

	_ChannelInfo* pChannelInfo = null;

	__channels.GetValue(dest, pChannelInfo);
	SysTryReturnResult(NID_IO, pChannelInfo != null, E_OBJ_NOT_FOUND,
				"Destination channel not found.");

	result r = __pIChannelServiceStub->SendResponse(pChannelInfo->clientId, src, dest, args, requestId);
	SysTryReturnResult(NID_IO, r == E_SUCCESS, E_SYSTEM, "Failed to send the response data");

	return E_SUCCESS;
}

result
_ChannelService::SendNullResponse(const String& src,
							  const String& dest,
							  int requestId)
{
	SysLog(NID_IO, "[%ls] ---> [%ls], Request = %d", src.GetPointer(), dest.GetPointer(), requestId);

	_ChannelInfo* pChannelInfo = null;

	__channels.GetValue(dest, pChannelInfo);
	SysTryReturnResult(NID_IO, pChannelInfo != null, E_OBJ_NOT_FOUND,
				"Destination channel not found.");

	__pIChannelServiceStub->SendNullResponse(pChannelInfo->clientId, src, dest, requestId);

	return E_SUCCESS;
}

bool
_ChannelService::IsChannelRegistered(const String& channelId)
{
	bool out = false;

	__channels.ContainsKey(channelId, out);

	return out;
}

}}
